@@ -5,11 +5,11 @@ use derive_new::new;
 use std::collections::{HashMap, VecDeque};
 use std::fs::{DirEntry, File};
 use std::io::Write;
-use std::path::PathBuf;
+use std::path::Path;
 use std::rc::Rc;
+use std::time::Instant;
 use syn::__private::ToTokens;
 use syn::{PathArguments, ReturnType, Type};
-use std::time::Instant;
 
 //constants
 
@@ -102,7 +102,7 @@ macro_rules! correct_string {
 
 macro_rules! types_in_method {
     ($expr:expr) => {{
-        let result = (&$expr)
+        let result = ($expr)
             .sig
             .inputs
             .iter()
@@ -241,10 +241,9 @@ impl ItemsHolder {
             let item_name = item.name().to_string();
             values.remove(&item_name);
             let item_name = Rc::new(item_name);
-            out.iter().position(|it| it == &item_name).and_then(|it| {
-                //println!("and then {}", item_name);
-                Some(out.remove(it))
-            });
+            out.iter()
+                .position(|it| it == &item_name)
+                .map(|it| out.remove(it));
             out.push_front(item_name);
             for _type in types {
                 //println!("type {}", _type);
@@ -271,9 +270,7 @@ impl ItemsHolder {
                 values.iter().next().unwrap().0.to_string()
             );*/
             analyse_item(
-                self.list
-                    .get(&values.iter().next().unwrap().0.to_string())
-                    .unwrap(),
+                self.list.get(values.iter().next().unwrap().0).unwrap(),
                 &mut values,
                 &self.list,
                 &mut self.final_list,
@@ -293,10 +290,10 @@ impl ItemsHolder {
         self.enums_list.push(data)
     }
 
-    fn generate_interface(mut self, language: Language, out_file: &PathBuf) {
+    fn generate_interface<P: AsRef<Path>>(mut self, language: Language, out_file: P) {
         //println!("final {:?}", self.final_list);
         let mut file = File::create(out_file).expect("Unable to write to disk");
-        file.write(b"//Automatically generated by rifgen\nuse crate::*;\n")
+        file.write_all(b"//Automatically generated by rifgen\nuse crate::*;\n")
             .unwrap();
         if matches!(language, Language::Java) {
             file.write_all(b"use jni_sys::*;\n").unwrap();
@@ -332,8 +329,11 @@ impl ItemsHolder {
     }
 }
 // one possible implementation of walking a directory only visiting files
-fn visit_dirs(dir: &PathBuf, cb: &mut dyn FnMut(&std::fs::DirEntry)) -> std::io::Result<()> {
-    if dir.is_dir() {
+fn visit_dirs<P: AsRef<Path>>(
+    dir: P,
+    cb: &mut dyn FnMut(&std::fs::DirEntry),
+) -> std::io::Result<()> {
+    if dir.as_ref().is_dir() {
         for entry in std::fs::read_dir(dir)? {
             let entry = entry?;
             let path = entry.path();
@@ -347,17 +347,17 @@ fn visit_dirs(dir: &PathBuf, cb: &mut dyn FnMut(&std::fs::DirEntry)) -> std::io:
     Ok(())
 }
 
-pub struct FileGenerator {
-    interface_file_path: PathBuf,
-    starting_point: PathBuf,
+pub struct FileGenerator<I: AsRef<Path>, S: AsRef<Path>> {
+    interface_file_path: I,
+    starting_point: S,
 }
 
-impl FileGenerator {
+impl<I: AsRef<Path>, S: AsRef<Path>> FileGenerator<I, S> {
     pub fn new(
         type_case: TypeCases,
-        interface_file_path: PathBuf,
-        starting_point: PathBuf,
-    ) -> FileGenerator {
+        interface_file_path: I,
+        starting_point: S,
+    ) -> FileGenerator<I, S> {
         unsafe { TYPE_CASE = type_case }
         FileGenerator {
             interface_file_path,
@@ -371,11 +371,8 @@ impl FileGenerator {
         let mut file_data: HashMap<Rc<String>, TypeHolder> = HashMap::new();
         let mut closure = |file: &DirEntry| {
             let file_path = file.path();
-            let file_contents = std::fs::read_to_string(&file_path).expect(&format!(
-                "{}{}",
-                UNABLE_TO_READ,
-                file_path.to_str().unwrap()
-            ));
+            let file_contents = std::fs::read_to_string(&file_path)
+                .unwrap_or_else(|_| panic!("{}{}", UNABLE_TO_READ, file_path.to_str().unwrap()));
             let compiled_file = syn::parse_file(&file_contents).expect("Invalid rust file");
 
             for item in &compiled_file.items {
@@ -384,7 +381,7 @@ impl FileGenerator {
                     syn::Item::Struct(item) => {
                         //check if it has the doc attribute
                         if has_doc_gen_attr!(item) {
-                            let name = Rc::new((&item).ident.to_string());
+                            let name = Rc::new(item.ident.to_string());
                             //assert!(!file_data.contains_key(&name.clone()));
                             //the impl block may come (ie if it's in a different file) before the struct definition
                             if let Some(val) = file_data.get_mut(&name) {
@@ -421,11 +418,11 @@ impl FileGenerator {
                     }
                     syn::Item::Impl(val) => {
                         //TODO let it work with enums
-                        FileGenerator::impl_data(&mut file_data, val);
+                        FileGenerator::<&Path, &Path>::impl_data(&mut file_data, val);
                     }
                     syn::Item::Enum(val) => {
                         if has_gen_attr!(val).is_attribute {
-                            let name = Rc::new((&val).ident.to_string());
+                            let name = Rc::new(val.ident.to_string());
                             assert!(
                                 !file_data.contains_key(&name),
                                 "Multiple definitions of {}",
@@ -453,7 +450,7 @@ impl FileGenerator {
                             continue;
                         }
                         //println!("trait");
-                        let name = Rc::new((&val).ident.to_string());
+                        let name = Rc::new(val.ident.to_string());
                         let mut trait_data: Trait = Trait::new(
                             name.to_string(),
                             Types::Trait,
@@ -462,7 +459,7 @@ impl FileGenerator {
                         );
                         for item in &val.items {
                             if let syn::TraitItem::Method(method) = item {
-                                let method_name = (&method).sig.ident.to_string();
+                                let method_name = method.sig.ident.to_string();
                                 trait_data.extras.push(ItemInfo::new_method(
                                     function_signature!(method),
                                     get_doc!(method),
@@ -510,51 +507,48 @@ impl FileGenerator {
                 .segments
                 .iter()
                 .next()
-                .and_then(|it| Some(it.ident.to_string()));
+                .map(|it| it.ident.to_string());
             if let Some(name) = name {
                 //name of struct or enum
                 for item in item.items.iter() {
-                    match item {
-                        syn::ImplItem::Method(method) => {
-                            let method_info: AttrCheck = has_gen_attr!(method, true);
-                            //not supporting enums for now
-                            if !method_info.is_attribute {
-                                continue;
+                    if let syn::ImplItem::Method(method) = item {
+                        let method_info: AttrCheck = has_gen_attr!(method, true);
+                        //not supporting enums for now
+                        if !method_info.is_attribute {
+                            continue;
+                        }
+                        let method_name = method.sig.ident.to_string();
+                        let data = map.get_mut(&name);
+                        let item_info = ItemInfo::new_method(
+                            function_signature!(method),
+                            get_doc!(method),
+                            method_name,
+                            method_info.is_constructor,
+                            types_in_method!(method),
+                            return_types!(method),
+                        );
+                        if let Some(data) = data {
+                            match data {
+                                TypeHolder::Struct(val) => {
+                                    val.extras.push(item_info);
+                                }
+                                _ => {
+                                    unimplemented!("Impl function may only be used for structs")
+                                }
                             }
-                            let method_name = (&method).sig.ident.to_string();
-                            let data = map.get_mut(&name);
-                            let item_info = ItemInfo::new_method(
-                                function_signature!(method),
-                                get_doc!(method),
-                                method_name,
-                                method_info.is_constructor,
-                                types_in_method!(method),
-                                return_types!(method),
+                        } else {
+                            //impl block came before struct definition (properly due to the order in which the files
+                            // were read)
+                            //we're assuming the impl method is for a struct
+                            //if it's for an enum, it would crash in the enum function
+                            let data = Struct::new(
+                                name.to_string(),
+                                Types::Struct,
+                                vec![],
+                                vec![item_info],
                             );
-                            if let Some(data) = data {
-                                match data {
-                                    TypeHolder::Struct(val) => {
-                                        val.extras.push(item_info);
-                                    }
-                                    _ => {
-                                        unimplemented!("Impl function may only be used for structs")
-                                    }
-                                }
-                            } else {
-                                //impl block came before struct definition (properly due to the order in which the files
-                                // were read)
-                                //we're assuming the impl method is for a struct
-                                //if it's for an enum, it would crash in the enum function
-                                let data = Struct::new(
-                                    name.to_string(),
-                                    Types::Struct,
-                                    vec![],
-                                    vec![item_info],
-                                );
-                                map.insert(Rc::new(name.clone()), TypeHolder::Struct(data));
-                            }
+                            map.insert(Rc::new(name.clone()), TypeHolder::Struct(data));
                         }
-                        _ => {}
                     }
                 }
             }
@@ -668,4 +662,4 @@ struct Kofi {
         }
     }
 }
-*/
\ No newline at end of file
+*/
